// Import the react state management
import { useEffect, useState } from 'react';

// Import the 
import { EditorSelection, EditorState, StateEffect, Transaction } from '@codemirror/state';
import { EditorView, highlightActiveLine } from '@codemirror/view';

// Add the gutter config
import { lineNumbers, gutter } from '@codemirror/gutter';

// Import the language
import { GCodeLanguage } from 'gcode-lang-codemirror';

// Import the editor
import { SelectedLine } from './extensions/SelectedLine.js';
import { LineSelectionController } from './extensions/LineSelectionController.js'

export function useCodeMirror(props) {

  // Extract the properties
  const { code, selectLine } = props;

  // console.log(props)

  const [container, setContainer] = useState(props.container);
  const [view, setView] = useState();
  const [state, setState] = useState();
  // const [selection, setSelection] = useState();

  const extensions = [
    GCodeLanguage(),
    highlightActiveLine(),
    lineNumbers(),
    gutter(),
    SelectedLine(selectLine),
    LineSelectionController()
  ];

  // Bind the update listener
  //   const updateListener = EditorView.updateListener.of(update => {
  //       if (update.docChanged && typeof onChange === 'function') {
  //           const doc = update.state.doc;
  //           const value = doc.toString();
  //           onChange(value, update);
  //       }
  //   });

  // Bind the selection listener
  //   let getExtensions = [updateListener, defaultThemeOption];
  //   if (basicSetup) {
  //       getExtensions.unshift(defaultBasicSetup);
  //   }
  //   if (placeholder) {
  //       getExtensions.unshift(extendPlaceholder(placeholder));
  //   }

  //   if (onUpdate && typeof onUpdate === 'function') {
  //       getExtensions.push(EditorView.updateListener.of(onUpdate));
  //   }
  //   getExtensions = getExtensions.concat(extensions);
  useEffect(() => {
    if (container && !state) {

      // Create
      const stateCurrent = EditorState.create({
        doc: code,
        extensions: extensions
      });

      setState(stateCurrent);
      if (!view) {
        const viewCurrent = new EditorView({
          state: stateCurrent,
          parent: container
        });
        setView(viewCurrent);
      }
    }
  }, [container, state]);


  useEffect(() => {
    return () => {
      if (view) {
        view.destroy();
      }
    };
  }, [view]);


  useEffect(() => {
    if (view) {
      const currentValue = view.state.doc.toString();
      if (code !== currentValue) {
        view.dispatch({
          changes: {
            from: 0,
            to: currentValue.length,
            insert: code || ''
          },
        });
      }
    }
  }, [code, view]);

  return { state, setState, view, setView, container, setContainer };
}