// Import the panel controls
import { showPanel } from '@codemirror/panel';

// Define the select line function to be dispatched
function selectLine (update, number) {

  // Find the line from a line number
  const line = update.state.doc.line(number);

  // Select the line dispatching the update
  const transaction = update.state.update({
    annotations: [{
      value: 'select'
    }],
    selection: { anchor: line.from }
  });

  // Dispatch the transaction and update the view
  return update.view.dispatch(transaction);
}

// Create a button (may need changes for react)
function createButton (properties) {
  const button = document.createElement('button');
  Object.keys(properties).forEach(key => button[key] = properties[key]);
  return button;
}

// Define the plugin panel for controls
function lineControls(view) {

  // Create a container for the buttons
  const container = document.createElement('div');

  // Return the panel
  return {
    dom: container,

    // Handle the update loop
    update(update) {

      // Extract the state
      const { state } = update;

      // Extract the doc and selection
      const { doc, selection } = state;

      // Compute the number of lines
      const lines = doc.lines;

      // Compute the current line number
      const line = doc.lineAt(selection.main.from).number;

      // console.log(line)

      // Clear the container
      container.innerHTML = '';

      // Create a forward line button
      let next = createButton({ id: 'next', textContent: ' Next Line |>' });

      // Create a forward line button
      let previous = createButton({ id: 'previous', textContent: '<| Previous Line ' });

      // Append the buttons
      container.append(previous, next);

      // Handle first line
      if (line == 1) {
        previous.disabled = true;
        next.addEventListener('click', () => selectLine(update, line + 1));

      // Handle last line
      } else if (line == lines) {
        next.disabled = true;
        previous.addEventListener('click', () => selectLine(update, line - 1));

      // Handle any other line
      } else {
        previous.addEventListener('click', () => selectLine(update, line - 1));
        next.addEventListener('click', () => selectLine(update, line + 1));
      }
    }
  }
}

// Define the panel extension
export function LineSelectionController() {
  //
  return showPanel.of(lineControls);
}