// Import the functionality from react
import React, { useEffect, useRef, useImperativeHandle } from 'react';

// Import the functionality from the codemirror instance
import { useCodeMirror } from './codeMirror.js';

// Create a codemirror instance
export const ReactCodeMirror = React.forwardRef((props, ref) => {

    // Extract the properties from the instance
    const { code = '', selectLine } = props;

    // console.log(selected)

    // Create the empty reference
    const editor = useRef(null);

    // Create an instance fo codemirror to be bound
    const { state, view, container, setContainer } = useCodeMirror({
        container: editor.current,
        code,
        selectLine
    });

    useImperativeHandle(ref, () => ({ editor: container, state, view }), [container, state, view]);

    useEffect(() => {
        setContainer(editor.current);
        return () => {
            if (view) {
                view.destroy();
            }
        };
    }, []);

    // Create a react element using the reference
    return <div id="editor" ref={editor} />;
});

ReactCodeMirror.displayName = 'CodeMirror'; 

export default ReactCodeMirror;